@@ -5,7 +5,7 @@ package cloudi
 //
 // MIT License
 //
-// Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2017-2021 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@ import (
 	"bytes"
 	"container/list"
 	"encoding/binary"
-	"erlang"
 	"fmt"
+	"github.com/okeuday/erlang_go/v2/erlang"
 	"math"
 	"net"
 	"os"
@@ -703,6 +703,11 @@ func (api *Instance) TimeoutTerminate() uint32 {
 	return api.timeoutTerminate
 }
 
+// PriorityDefault returns the default service request send priority from the service configuration
+func (api *Instance) PriorityDefault() int8 {
+	return api.priorityDefault
+}
+
 func (api *Instance) callback(command uint32, name, pattern string, requestInfo, request []byte, timeout uint32, priority int8, transId [16]byte, pid Source) error {
 	functionQueue := api.callbacks[pattern]
 	var function Callback
