@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -140,7 +140,10 @@ public class API
     {
         final String protocol = System.getenv("CLOUDI_API_INIT_PROTOCOL");
         if (protocol == null)
+        {
+            API.err.println("CloudI service execution must occur in CloudI");
             throw new InvalidInputException();
+        }
         final String buffer_size_str =
             System.getenv("CLOUDI_API_INIT_BUFFER_SIZE");
         if (buffer_size_str == null)
@@ -1027,6 +1030,11 @@ public class API
         return this.timeout_terminate;
     }
 
+    public byte priority_default()
+    {
+        return this.priority_default;
+    }
+
     private void callback(final int command,
                           final String name,
                           final String pattern,
@@ -1126,6 +1134,11 @@ public class API
                 e_forward.printStackTrace(API.err);
                 return;
             }
+            catch (Error e)
+            {
+                e.printStackTrace(API.err);
+                System.exit(1);
+            }
             catch (Throwable e)
             {
                 e.printStackTrace(API.err);
@@ -1216,6 +1229,11 @@ public class API
                 e_forward.printStackTrace(API.err);
                 return;
             }
+            catch (Error e)
+            {
+                e.printStackTrace(API.err);
+                System.exit(1);
+            }
             catch (Throwable e)
             {
                 e.printStackTrace(API.err);
@@ -1625,6 +1643,28 @@ public class API
         return text.toByteArray();
     }
 
+    private static byte[] text_pairs_new(final HashMap<String,
+                                                       ArrayList<String>> pairs,
+                                         final boolean response)
+    {
+        final ByteArrayOutputStream text = new ByteArrayOutputStream(1024);
+        for (HashMap.Entry<String, ArrayList<String>> pair : pairs.entrySet())
+        {
+            final byte[] key_bytes = pair.getKey().getBytes();
+            for (String value : pair.getValue())
+            {
+                final byte[] value_bytes = value.getBytes();
+                text.write(key_bytes, 0, key_bytes.length);
+                text.write(0);
+                text.write(value_bytes, 0, value_bytes.length);
+                text.write(0);
+            }
+        }
+        if (response && text.size() == 0)
+            text.write(0);
+        return text.toByteArray();
+    }
+
     /**
      * Decode service request info key/value data
      *
@@ -1648,6 +1688,18 @@ public class API
         return API.info_key_value_new(pairs, true);
     }
 
+    /**
+     * Encode service response info key/value data
+     *
+     * @param  pairs info key/value map
+     * @return encoded binary
+     */
+    public static byte[] info_key_value_new(HashMap<String,
+                                                    ArrayList<String>> pairs)
+    {
+        return API.info_key_value_new(pairs, true);
+    }
+
     /**
      * Encode service response info key/value data
      *
@@ -1661,6 +1713,20 @@ public class API
         return API.text_pairs_new(pairs, response);
     }
 
+    /**
+     * Encode service response info key/value data
+     *
+     * @param  pairs info key/value map
+     * @param  response if encoding response data
+     * @return encoded binary
+     */
+    public static byte[] info_key_value_new(HashMap<String,
+                                                    ArrayList<String>> pairs,
+                                            boolean response)
+    {
+        return API.text_pairs_new(pairs, response);
+    }
+
     private void send(final OtpOutputStream command)
     {
         try
@@ -2018,7 +2084,7 @@ public class API
     public static class InvalidInputException extends Exception
     {
         private static final long serialVersionUID = 1L;
-        InvalidInputException()
+        private InvalidInputException()
         {
             super("Invalid Input");
         }
@@ -2027,7 +2093,7 @@ public class API
     public static class ReturnSyncException extends Exception
     {
         private static final long serialVersionUID = 3L;
-        ReturnSyncException()
+        private ReturnSyncException()
         {
             super("Synchronous Call Return Invalid");
         }
@@ -2036,7 +2102,7 @@ public class API
     public static class ReturnAsyncException extends Exception
     {
         private static final long serialVersionUID = 3L;
-        ReturnAsyncException()
+        private ReturnAsyncException()
         {
             super("Asynchronous Call Return Invalid");
         }
@@ -2045,7 +2111,7 @@ public class API
     public static class ForwardSyncException extends Exception
     {
         private static final long serialVersionUID = 3L;
-        ForwardSyncException()
+        private ForwardSyncException()
         {
             super("Synchronous Call Forward Invalid");
         }
@@ -2054,7 +2120,7 @@ public class API
     public static class ForwardAsyncException extends Exception
     {
         private static final long serialVersionUID = 3L;
-        ForwardAsyncException()
+        private ForwardAsyncException()
         {
             super("Asynchronous Call Forward Invalid");
         }
@@ -2063,7 +2129,7 @@ public class API
     public static class MessageDecodingException extends Exception
     {
         private static final long serialVersionUID = 1L;
-        MessageDecodingException()
+        private MessageDecodingException()
         {
             super("Message Decoding Error");
         }
@@ -2073,7 +2139,7 @@ public class API
     {
         private static final long serialVersionUID = 0L;
         private int timeout;
-        TerminateException(final int timeout)
+        private TerminateException(final int timeout)
         {
             super("Terminate");
             this.timeout = timeout;
@@ -2084,5 +2150,14 @@ public class API
         }
     }
 
+    public static class FatalError extends Error
+    {
+        private static final long serialVersionUID = 0L;
+        public FatalError(final String message)
+        {
+            super(message);
+        }
+    }
+
 }
 
