@@ -3,7 +3,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2014-2021 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -29,7 +29,7 @@ use strict;
 use warnings;
 use 5.010;
 
-$CloudI::API::VERSION = '2.01';
+$CloudI::API::VERSION = '2.04';
 
 use POSIX qw(getenv);
 use IO::Handle;
@@ -464,6 +464,12 @@ sub timeout_terminate
     return $self->{_timeout_terminate};
 }
 
+sub priority_default
+{
+    my $self = shift;
+    return $self->{_priority_default};
+}
+
 sub _null_response
 {
     my $self = shift;
