@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -57,6 +57,7 @@ module CloudI
                 @s.sync = true
                 @use_header = true
             else
+                $stderr.puts 'CloudI service execution must occur in CloudI'
                 raise InvalidInputException
             end
             @initialization_complete = false
@@ -76,6 +77,7 @@ module CloudI
         attr_reader :timeout_async
         attr_reader :timeout_sync
         attr_reader :timeout_terminate
+        attr_reader :priority_default
 
         def self.thread_count
             s = getenv('CLOUDI_API_INIT_THREAD_COUNT')
@@ -182,7 +184,7 @@ module CloudI
                                         OtpErlangBinary.new(request),
                                         timeout, priority,
                                         OtpErlangBinary.new(trans_id), pid]))
-            raise ForwardAsyncException.new()
+            raise ForwardAsyncException
         end
 
         def forward_sync(name, request_info, request,
@@ -192,7 +194,7 @@ module CloudI
                                         OtpErlangBinary.new(request),
                                         timeout, priority,
                                         OtpErlangBinary.new(trans_id), pid]))
-            raise ForwardSyncException.new()
+            raise ForwardSyncException
         end
 
         def return_(request_type, name, pattern, response_info, response,
@@ -214,7 +216,7 @@ module CloudI
                                         OtpErlangBinary.new(response),
                                         timeout,
                                         OtpErlangBinary.new(trans_id), pid]))
-            raise ReturnAsyncException.new()
+            raise ReturnAsyncException
         end
 
         def return_sync(name, pattern, response_info, response,
@@ -224,7 +226,7 @@ module CloudI
                                         OtpErlangBinary.new(response),
                                         timeout,
                                         OtpErlangBinary.new(trans_id), pid]))
-            raise ReturnSyncException.new()
+            raise ReturnSyncException
         end
 
         def recv_async(timeout=nil, trans_id=nil, consume=true)
@@ -293,10 +295,18 @@ module CloudI
                     $stderr.puts e.message
                     $stderr.puts e.backtrace
                     return
-                rescue
+                rescue StandardError => e
                     return_null_response = true
+                    $stderr.puts e.message
+                    $stderr.puts e.backtrace
+                rescue SystemExit => e
+                    $stderr.puts e.message
+                    $stderr.puts e.backtrace
+                    raise
+                rescue
                     $stderr.puts $!.message
                     $stderr.puts $!.backtrace
+                    exit(1)
                 end
                 if return_null_response
                     response_info = ''
@@ -344,10 +354,18 @@ module CloudI
                     $stderr.puts e.message
                     $stderr.puts e.backtrace
                     return
-                rescue
+                rescue StandardError => e
                     return_null_response = true
+                    $stderr.puts e.message
+                    $stderr.puts e.backtrace
+                rescue SystemExit => e
+                    $stderr.puts e.message
+                    $stderr.puts e.backtrace
+                    raise
+                rescue
                     $stderr.puts $!.message
                     $stderr.puts $!.backtrace
+                    exit(1)
                 end
                 if return_null_response
                     response_info = ''
@@ -679,7 +697,7 @@ module CloudI
         end
 
         def self.assert
-            raise 'Assertion failed !' unless yield # if $DEBUG
+            raise AssertionError unless yield # if $DEBUG
         end
 
         private :null_response
@@ -773,5 +791,11 @@ module CloudI
         end
         attr_reader :timeout
     end
+
+    class AssertionError < Exception
+    end
+
+    class FatalError < Exception
+    end
 end
 
