@@ -270,27 +270,36 @@ handle_call({new_entry, Name, Type, Opts, AllowExisting} = _Req, _From, S) ->
         #cloudi_x_exometer_entry{options = NewOpts} = E0 =
             lookup_definition(Name, Type, Opts),
 
-        case {ets:member(cloudi_x_exometer_util:table(), Name), AllowExisting} of
-            {true, false} ->
+        case {ets:lookup(cloudi_x_exometer_util:table(), Name), AllowExisting} of
+            {[_], false} ->
                 {reply, {error, exists}, S};
-            _Other ->
-		?log(debug, "_Other = ~p~n", [_Other]),
+            {LookupRes, _} ->
+                ?log(debug, "LookupRes = ~p~n", [LookupRes]),
                 E1 = process_opts(E0, NewOpts),
-                Res = try  cloudi_x_exometer:create_entry(E1),
-			   cloudi_x_exometer_report:new_entry(E1)
-		      catch
-			  ?EXCEPTION(error, Error1, Stacktrace1) ->
-			      ?log(debug,
-				"ERROR create_entry(~p) :- ~p~n~p",
-				[E1, Error1, ?GET_STACK(Stacktrace1)]),
-			      erlang:error(Error1)
-		      end,
+                try
+                   remove_old_instance(LookupRes, Name)
+                catch
+                    ?EXCEPTION(Cat, Exception, Stacktrace1) ->
+                        ?log(debug, "CAUGHT(~p) ~p:~p / ~p",
+                             [Name, Cat, Exception, ?GET_STACK(Stacktrace1)]),
+                        ok
+                end,
+                Res = try
+                          cloudi_x_exometer:create_entry(E1),
+                          cloudi_x_exometer_report:new_entry(E1)
+                      catch
+                          ?EXCEPTION(error, Error1, Stacktrace2) ->
+                              ?log(debug,
+                                   "ERROR create_entry(~p) :- ~p~n~p",
+                                   [E1, Error1, ?GET_STACK(Stacktrace2)]),
+                              erlang:error(Error1)
+                      end,
                 {reply, Res, S}
         end
     catch
         ?EXCEPTION(error, Error, Stacktrace) ->
-	    ?log(error, "~p -*-> error:~p~n~p~n",
-			[_Req, Error, ?GET_STACK(Stacktrace)]),
+            ?log(error, "~p -*-> error:~p~n~p~n",
+                 [_Req, Error, ?GET_STACK(Stacktrace)]),
             {reply, {error, Error}, S}
     end;
 handle_call({repair_entry, Name}, _From, S) ->
@@ -691,32 +700,9 @@ try_delete_entry_(Name) ->
 delete_entry_(Name) ->
     cloudi_x_exometer_cache:delete_name(Name),
     case ets:lookup(cloudi_x_exometer_util:table(), Name) of
-        [#cloudi_x_exometer_entry{module = cloudi_x_exometer, type = Type}] when Type==counter;
-                                                               Type==gauge ->
-            [ets:delete(T, Name) ||
-                T <- [?EXOMETER_ENTRIES|cloudi_x_exometer_util:tables()]],
-            ok;
-        [#cloudi_x_exometer_entry{module = cloudi_x_exometer, type = fast_counter,
-                         ref = {M, F}}] ->
+        [#cloudi_x_exometer_entry{} = Entry] ->
             try
-                cloudi_x_exometer_util:set_call_count(M, F, false)
-            after
-                [ets:delete(T, Name) ||
-                    T <- [?EXOMETER_ENTRIES|cloudi_x_exometer_util:tables()]]
-            end,
-            ok;
-        [#cloudi_x_exometer_entry{behaviour = probe,
-                         type = Type, ref = Ref}] ->
-            try
-                cloudi_x_exometer_probe:delete(Name, Type, Ref)
-            after
-                [ets:delete(T, Name) ||
-                    T <- [?EXOMETER_ENTRIES|cloudi_x_exometer_util:tables()]]
-            end,
-            ok;
-        [#cloudi_x_exometer_entry{module= Mod, behaviour = entry,
-                         type = Type, ref = Ref}] ->
-            try Mod:delete(Name, Type, Ref)
+                remove_old_instance(Entry, Name)
             after
                 [ets:delete(T, Name) ||
                     T <- [?EXOMETER_ENTRIES|cloudi_x_exometer_util:tables()]]
@@ -725,3 +711,19 @@ delete_entry_(Name) ->
         [] ->
             {error, not_found}
     end.
+
+remove_old_instance([], _) ->
+    ok;
+remove_old_instance([Entry], Name) ->
+    remove_old_instance(Entry, Name);
+remove_old_instance(#cloudi_x_exometer_entry{module = cloudi_x_exometer, type = fast_counter,
+                                    ref = {M, F}}, _) ->
+    cloudi_x_exometer_util:set_call_count(M, F, false);
+remove_old_instance(#cloudi_x_exometer_entry{behaviour = probe,
+                                    type = Type, ref = Ref}, Name) ->
+    cloudi_x_exometer_probe:delete(Name, Type, Ref);
+remove_old_instance(#cloudi_x_exometer_entry{module = Mod, behaviour = entry,
+                                    type = Type, ref = Ref}, Name) ->
+    Mod:delete(Name, Type, Ref);
+remove_old_instance(_, _) ->
+    ok.
