@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2020 Michael Truog
-%%% @version 2.0.1 {@date} {@time}
+%%% @copyright 2011-2021 Michael Truog
+%%% @version 2.0.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_nodes).
@@ -725,7 +725,7 @@ node_status_cost(CostValue, NanoSeconds, CostPrecision, LocalNode, Status0) ->
       erlang:float_to_list(CostCurrency,
                            [{decimals, CostPrecision}])} | StatusN].
 
-discovery_start_args(ec2_discover, StartA) ->
+discovery_start_args(ec2_start, StartA) ->
     [EC2AccessKeyId, EC2SecretAccessKey,
      EC2Host, EC2Groups, EC2Tags] = StartA,
     Environment = cloudi_environment:lookup(),
