@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2020 Michael Truog
-%%% @version 2.0.1 {@date} {@time}
+%%% @copyright 2011-2021 Michael Truog
+%%% @version 2.0.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_api).
@@ -336,6 +336,24 @@
               aspect_terminate_before_external_f/0,
               aspect_terminate_before_internal/0,
               aspect_terminate_before_external/0]).
+-type aspect_suspend_f() ::
+    fun((State :: any()) ->
+        {ok, State :: any()}).
+-type aspect_suspend() ::
+    aspect_suspend_f() |
+    {Module :: module(), Function :: atom()} |
+    {{Module :: module(), Function :: atom()}}.
+-type aspect_resume_f() ::
+    fun((State :: any()) ->
+        {ok, State :: any()}).
+-type aspect_resume() ::
+    aspect_resume_f() |
+    {Module :: module(), Function :: atom()} |
+    {{Module :: module(), Function :: atom()}}.
+-export_type([aspect_suspend_f/0,
+              aspect_suspend/0,
+              aspect_resume_f/0,
+              aspect_resume/0]).
 
 -type max_heap_size_options() ::
     non_neg_integer() |
@@ -451,13 +469,15 @@
                {min_bin_vheap_size, non_neg_integer()} |
                {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
-               {message_queue_data, off_heap | on_heap | mixed})} |
+               {message_queue_data, off_heap | on_heap})} |
          {aspects_init_after, list(aspect_init_after_internal())} |
          {aspects_request_before, list(aspect_request_before_internal())} |
          {aspects_request_after, list(aspect_request_after_internal())} |
          {aspects_info_before, list(aspect_info_before_internal())} |
          {aspects_info_after, list(aspect_info_after_internal())} |
          {aspects_terminate_before, list(aspect_terminate_before_internal())} |
+         {aspects_suspend, list(aspect_suspend())} |
+         {aspects_resume, list(aspect_resume())} |
          {application_name, undefined | atom()} |
          {init_pid_options,
           list({priority, low | normal | high} |
@@ -466,7 +486,7 @@
                {min_bin_vheap_size, non_neg_integer()} |
                {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
-               {message_queue_data, off_heap | on_heap | mixed})} |
+               {message_queue_data, off_heap | on_heap})} |
          {request_pid_uses, infinity | pos_integer()} |
          {request_pid_options,
           list({priority, low | normal | high} |
@@ -475,7 +495,7 @@
                {min_bin_vheap_size, non_neg_integer()} |
                {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
-               {message_queue_data, off_heap | on_heap | mixed})} |
+               {message_queue_data, off_heap | on_heap})} |
          {info_pid_uses, infinity | pos_integer()} |
          {info_pid_options,
           list({priority, low | normal | high} |
@@ -484,7 +504,7 @@
                {min_bin_vheap_size, non_neg_integer()} |
                {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
-               {message_queue_data, off_heap | on_heap | mixed})} |
+               {message_queue_data, off_heap | on_heap})} |
          {duo_mode, boolean()} |
          {hibernate,
           service_options_internal_hibernate_options() | boolean()} |
@@ -524,11 +544,13 @@
                {min_bin_vheap_size, non_neg_integer()} |
                {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
-               {message_queue_data, off_heap | on_heap | mixed})} |
+               {message_queue_data, off_heap | on_heap})} |
          {aspects_init_after, list(aspect_init_after_external())} |
          {aspects_request_before, list(aspect_request_before_external())} |
          {aspects_request_after, list(aspect_request_after_external())} |
          {aspects_terminate_before, list(aspect_terminate_before_external())} |
+         {aspects_suspend, list(aspect_suspend())} |
+         {aspects_resume, list(aspect_resume())} |
          {limit, limit_external()} |
          {owner, owner_external()} |
          {nice, nice_external()} |
@@ -602,6 +624,9 @@
 % `"0"'
 -type integer_string_ge_0() ::
     nonempty_list($0..$9).
+% `"11"'
+-type integer_string_gt_0() ::
+    nonempty_list($0..$9).
 % `"0.0"'
 -type float_string_ge_0() ::
     nonempty_list($0..$9 | $.).
@@ -614,6 +639,7 @@
               seconds_string/0,
               seconds_string_signed/0,
               integer_string_ge_0/0,
+              integer_string_gt_0/0,
               float_string_ge_0/0]).
 
 -type service_status_internal() ::
@@ -621,6 +647,7 @@
                   {prefix, cloudi:service_name_pattern()} |
                   {module, atom()} |
                   {count_process, pos_integer()} |
+                  {pids_erlang, list(pid())} |
                   {suspended, boolean()} |
                   {uptime_total, nanoseconds_string()} |
                   {uptime_running, nanoseconds_string()} |
@@ -660,6 +687,8 @@
                   {file_path, file_path()} |
                   {count_process, pos_integer()} |
                   {count_thread, pos_integer()} |
+                  {pids_os, list(pos_integer())} |
+                  {pids_erlang, list(pid())} |
                   {suspended, boolean()} |
                   {uptime_total, nanoseconds_string()} |
                   {uptime_running, nanoseconds_string()} |
@@ -769,13 +798,15 @@
                {min_bin_vheap_size, non_neg_integer()} |
                {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
-               {message_queue_data, off_heap | on_heap | mixed})} |
+               {message_queue_data, off_heap | on_heap})} |
          {aspects_init_after, list(aspect_init_after_internal())} |
          {aspects_request_before, list(aspect_request_before_internal())} |
          {aspects_request_after, list(aspect_request_after_internal())} |
          {aspects_info_before, list(aspect_info_before_internal())} |
          {aspects_info_after, list(aspect_info_after_internal())} |
          {aspects_terminate_before, list(aspect_terminate_before_internal())} |
+         {aspects_suspend, list(aspect_suspend())} |
+         {aspects_resume, list(aspect_resume())} |
          {init_pid_options,
           list({priority, low | normal | high} |
                {fullsweep_after, non_neg_integer()} |
@@ -783,7 +814,7 @@
                {min_bin_vheap_size, non_neg_integer()} |
                {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
-               {message_queue_data, off_heap | on_heap | mixed})} |
+               {message_queue_data, off_heap | on_heap})} |
          {request_pid_uses, infinity | pos_integer()} |
          {request_pid_options,
           list({priority, low | normal | high} |
@@ -792,7 +823,7 @@
                {min_bin_vheap_size, non_neg_integer()} |
                {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
-               {message_queue_data, off_heap | on_heap | mixed})} |
+               {message_queue_data, off_heap | on_heap})} |
          {info_pid_uses, infinity | pos_integer()} |
          {info_pid_options,
           list({priority, low | normal | high} |
@@ -801,7 +832,7 @@
                {min_bin_vheap_size, non_neg_integer()} |
                {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
-               {message_queue_data, off_heap | on_heap | mixed})} |
+               {message_queue_data, off_heap | on_heap})} |
          {hibernate,
           list({period, period_seconds()} |
                {rate_request_min, number()}) | boolean()} |
@@ -832,12 +863,13 @@
                {min_bin_vheap_size, non_neg_integer()} |
                {max_heap_size, max_heap_size_options()} |
                {sensitive, boolean()} |
-               {message_queue_data, off_heap | on_heap | mixed})} |
+               {message_queue_data, off_heap | on_heap})} |
          {aspects_init_after, list(aspect_init_after_external())} |
          {aspects_request_before, list(aspect_request_before_external())} |
          {aspects_request_after, list(aspect_request_after_external())} |
          {aspects_terminate_before, list(aspect_terminate_before_external())} |
-         {limit, limit_external()}).
+         {aspects_suspend, list(aspect_suspend())} |
+         {aspects_resume, list(aspect_resume())}).
 -type service_update_plan() ::
     service_update_plan_internal() |
     service_update_plan_external().
@@ -876,7 +908,10 @@
     nonempty_list({set, all | local} |
                   nodes_properties()).
 -type node_status_local() ::
-    nonempty_list({uptime, nanoseconds_string()} |
+    nonempty_list({services_running, integer_string_ge_0()} |
+                  {services_restarted, integer_string_ge_0()} |
+                  {services_failed, integer_string_ge_0()} |
+                  {uptime, nanoseconds_string()} |
                   {uptime_cost_total, float_string_ge_0()} |
                   {uptime_cost_day, float_string_ge_0()} |
                   {uptime_cost_week, float_string_ge_0()} |
@@ -924,7 +959,7 @@
          Line :: pos_integer(),
          Function :: atom() | undefined,
          Arity :: arity() | undefined,
-         MetaData :: list({atom(), any()}),
+         MetaData :: list({atom(), any()}) | #{},
          LogMessage :: iodata()) ->
         ok).
 -type aspect_log_before() ::
@@ -976,23 +1011,48 @@
                {output_max_t, cloudi_service_api:seconds()} |
                {formatter, module() | undefined} |
                {formatter_config, list()})}).
+-type iso8601() ::
+    cloudi_timestamp:iso8601().
+-type iso8601_seconds() ::
+    cloudi_timestamp:iso8601_seconds().
+% `"+3.104690100 seconds"'
+-type seconds_change_string() ::
+    nonempty_list($a..$z | $0..$9 | $. | $  | $+ | $-).
 -type logging_status() ::
-    list({file_messages_fatal, nonempty_string()} |
-         {file_messages_error, nonempty_string()} |
-         {file_messages_warn, nonempty_string()} |
-         {file_messages_info, nonempty_string()} |
-         {file_messages_debug, nonempty_string()} |
-         {file_messages_trace, nonempty_string()} |
-         {file_sync_fail_count, nonempty_string()} |
+    list({queue_mode, async | sync | overload} |
+         {queue_mode_sync_last_start, iso8601()} |
+         {queue_mode_sync_last_start_event, iso8601()} |
+         {queue_mode_sync_last_end, iso8601()} |
+         {queue_mode_sync_last_end_event, iso8601()} |
+         {queue_mode_sync_last_total, nanoseconds_string()} |
+         {queue_mode_overload_last_start, iso8601()} |
+         {queue_mode_overload_last_start_event, iso8601()} |
+         {queue_mode_overload_last_end, iso8601()} |
+         {queue_mode_overload_last_end_event, iso8601()} |
+         {queue_mode_overload_last_total, nanoseconds_string()} |
+         {time_offset_last_change, seconds_change_string()} |
+         {time_offset_last_event, iso8601()} |
+         {file_messages_fatal, integer_string_gt_0()} |
+         {file_messages_error, integer_string_gt_0()} |
+         {file_messages_warn, integer_string_gt_0()} |
+         {file_messages_info, integer_string_gt_0()} |
+         {file_messages_debug, integer_string_gt_0()} |
+         {file_messages_trace, integer_string_gt_0()} |
+         {file_sync_fail_count, integer_string_gt_0()} |
          {file_sync_fail_types, nonempty_list(atom())} |
-         {file_write_fail_count, nonempty_string()} |
-         {file_write_fail_types, nonempty_list(atom())}).
+         {file_write_fail_count, integer_string_gt_0()} |
+         {file_write_fail_types, nonempty_list(atom())} |
+         {file_read_fail_count, integer_string_gt_0()} |
+         {file_read_fail_types, nonempty_list(atom())}).
 -type logging_proplist() ::
     nonempty_list({file, string() | undefined} |
-                  {stdout, boolean()} |
                   {level, loglevel()} |
                   {redirect, node() | undefined} |
                   {syslog, logging_syslog_set_proplist() | undefined} |
+                  {stdout, boolean()} |
+                  {queue_mode_async, pos_integer()} |
+                  {queue_mode_sync, pos_integer()} |
+                  {queue_mode_overload, pos_integer()} |
                   {formatters, logging_formatters_set_proplist() | undefined} |
                   {log_time_offset, loglevel()} |
                   {aspects_log_before, list(aspect_log_before())} |
@@ -1009,13 +1069,10 @@
               logging_syslog_port/0,
               logging_syslog_set_proplist/0,
               logging_formatters_set_proplist/0,
+              seconds_change_string/0,
               logging_status/0,
               logging_proplist/0]).
 
--type iso8601() ::
-    cloudi_timestamp:iso8601().
--type iso8601_seconds() ::
-    cloudi_timestamp:iso8601_seconds().
 -type code_status_runtime_change() ::
     nonempty_list({type, internal | external} |
                   {file_age, seconds_string()} |
@@ -1041,6 +1098,7 @@
                   {runtime_erlang_stdlib_version, nonempty_string()} |
                   {runtime_erlang_sasl_version, nonempty_string()} |
                   {runtime_erlang_compiler_version, nonempty_string()} |
+                  {runtime_erlang_compilation, nonempty_string()} |
                   {runtime_cloudi_version, nonempty_string()} |
                   {runtime_machine_processors, pos_integer()} |
                   {runtime_start, iso8601()} |
@@ -1136,7 +1194,7 @@ acl(Timeout)
 %% When a subscription on the same service name pattern occurred
 %% multiple times, only a single entry is returned within the list.
 %% Service name patterns that are subscriptions of non-service Erlang pids
-%% (e.g., cloudi_service_http_cowboy1 websocket connection pids) will not
+%% (e.g., cloudi_service_http_cowboy websocket connection pids) will not
 %% be returned by this function.
 %% @end
 %%-------------------------------------------------------------------------
@@ -1318,7 +1376,7 @@ services_resume([_ | _] = L, Timeout)
 %% Multiple services may be returned for a single service name.  Only service
 %% instances on the local Erlang node are searched.  Service names that match
 %% subscriptions of non-service Erlang pids only
-%% (e.g., cloudi_service_http_cowboy1 websocket connection pids) will not
+%% (e.g., cloudi_service_http_cowboy websocket connection pids) will not
 %% return the service's configuration with this function.  Provide a scope
 %% within a 2 element tuple with the service name to check a custom scope.
 %% @end
@@ -1345,7 +1403,7 @@ services_search(Name, Timeout)
         [_ | _] = ServiceNameValue ->
             {?SCOPE_DEFAULT, ServiceNameValue}
     end,
-    try cloudi_x_trie:is_pattern2(ServiceName) of
+    try cloudi_x_trie:is_pattern2_bytes(ServiceName) of
         false ->
             cloudi_core_i_configurator:services_search(Scope, ServiceName,
                                                        Timeout);
@@ -1574,7 +1632,12 @@ nodes_status(L, Timeout)
            (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
            (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
-    cloudi_core_i_nodes:status(L, Timeout).
+    case cloudi_core_i_nodes:status(L, Timeout) of
+        {ok, NodesStatus} ->
+            nodes_status_local_services(NodesStatus, Timeout);
+        {error, _} = Error ->
+            Error
+    end.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -1925,6 +1988,23 @@ code_status(Timeout)
 %%% Private functions
 %%%------------------------------------------------------------------------
 
+nodes_status_local_services(NodesStatus, Timeout) ->
+    nodes_status_local_services(NodesStatus, [], Timeout).
+
+nodes_status_local_services([], L, _) ->
+    {ok, lists:reverse(L)};
+nodes_status_local_services([{Node, StatusList} | NodesStatus], L, Timeout)
+    when Node =:= node() ->
+    case cloudi_core_i_services_monitor:node_status(Timeout) of
+        {ok, LocalStatusList} ->
+            {ok, lists:reverse(L, [{Node, LocalStatusList ++ StatusList} |
+                                   NodesStatus])};
+        {error, _} = Error ->
+            Error
+    end;
+nodes_status_local_services([NodeStatus | NodesStatus], L, Timeout) ->
+    nodes_status_local_services(NodesStatus, [NodeStatus | L], Timeout).
+
 service_ids_convert_update(L) ->
     service_ids_convert_update(L, []).
 
