@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2020 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2015-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2015-2020 Michael Truog
-%%% @version 2.0.1 {@date} {@time}
+%%% @copyright 2015-2021 Michael Truog
+%%% @version 2.0.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_timestamp).
@@ -38,6 +38,7 @@
 
 %% external interface
 -export([convert/3,
+         datetime_utc/1,
          datetime_utc_to_string/1,
          timestamp/0,
          native/0,
@@ -57,6 +58,8 @@
          microseconds_epoch_to_string/1,
          microseconds_monotonic/0,
          microseconds_os/0,
+         microseconds_to_string/1,
+         microseconds_to_string/2,
          nanoseconds/0,
          nanoseconds_monotonic/0,
          nanoseconds_os/0,
@@ -83,6 +86,12 @@
 -type microseconds_monotonic() :: integer().
 -type nanoseconds_monotonic() :: integer().
 -type native_monotonic() :: integer().
+% `"1 days 5 hours 1563 seconds 557199 microseconds"'
+-type microseconds_string() ::
+    nonempty_list($a..$z | $0..$9 | $ ).
+% `"+ 110 microseconds"'
+-type microseconds_string_signed() ::
+    nonempty_list($a..$z | $0..$9 | $  | $+ | $-).
 % timestamps as 1970-01-01T00:00:00.000000Z
 %            or 1970-01-01T00:00:00.000Z
 -type iso8601() ::
@@ -100,6 +109,8 @@
               microseconds_monotonic/0,
               nanoseconds_monotonic/0,
               native_monotonic/0,
+              microseconds_string/0,
+              microseconds_string_signed/0,
               iso8601/0,
               iso8601_seconds/0]).
 
@@ -152,6 +163,28 @@ convert(Time, FromUnit, ToUnit) ->
     erlang:convert_time_unit(Time, FromUnitDeprecated, ToUnitDeprecated).
 -endif.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Create a datetime in UTC from native time units.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec datetime_utc(TimeNativeNow :: integer()) ->
+    calendar:datetime().
+
+-ifdef(ERLANG_OTP_VERSION_21_FEATURES).
+datetime_utc(TimeNativeNow) ->
+    calendar:system_time_to_universal_time(TimeNativeNow, native).
+-else.
+datetime_utc(TimeNativeNow) ->
+    TotalMicroSeconds = convert(TimeNativeNow, native, microsecond),
+    TotalSeconds = TotalMicroSeconds div 1000000,
+    MegaSeconds = TotalSeconds div 1000000,
+    Seconds = TotalSeconds - MegaSeconds * 1000000,
+    MicroSeconds = TotalMicroSeconds - TotalSeconds * 1000000,
+    calendar:now_to_universal_time({MegaSeconds, Seconds, MicroSeconds}).
+-endif.
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Create an ISO8601 timestamp from a datetime in UTC.===
@@ -469,6 +502,87 @@ microseconds_os() ->
     os:system_time(micro_seconds).
 -endif.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Format a microseconds duration as a minimal string with lower-precision integers.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec microseconds_to_string(TotalMicroSeconds :: non_neg_integer()) ->
+    microseconds_string().
+
+microseconds_to_string(TotalMicroSeconds)
+    when is_integer(TotalMicroSeconds), TotalMicroSeconds >= 0 ->
+    MicroSeconds = TotalMicroSeconds rem ?MICROSECONDS_IN_SECOND,
+    TotalSeconds = TotalMicroSeconds div ?MICROSECONDS_IN_SECOND,
+    Seconds = TotalSeconds rem ?SECONDS_IN_HOUR,
+    TotalHours = TotalSeconds div ?SECONDS_IN_HOUR,
+    Hours = TotalHours rem ?HOURS_IN_DAY,
+    TotalDays = TotalHours div ?HOURS_IN_DAY,
+    if
+        TotalDays > 0 ->
+            lists:flatten([time_value_to_list(TotalDays, "day"), $ ,
+                           time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(MicroSeconds, "microsecond")]);
+        Hours > 0 ->
+            lists:flatten([time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(MicroSeconds, "microsecond")]);
+        Seconds > 0 ->
+            lists:flatten([time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(MicroSeconds, "microsecond")]);
+        true ->
+            lists:flatten(time_value_to_list(MicroSeconds, "microsecond"))
+    end.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Format a microseconds duration with options as a minimal string with lower-precision integers.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec microseconds_to_string(TotalMicroSeconds :: integer(),
+                             Options :: signed) ->
+    microseconds_string_signed().
+
+microseconds_to_string(TotalMicroSeconds, signed)
+    when is_integer(TotalMicroSeconds) ->
+    {Sign, TotalMicroSecondsPositive} = if
+        TotalMicroSeconds >= 0 ->
+            {"+ ", TotalMicroSeconds};
+        true ->
+            {"- ", -1 * TotalMicroSeconds}
+    end,
+    MicroSeconds = TotalMicroSecondsPositive rem ?MICROSECONDS_IN_SECOND,
+    TotalSeconds = TotalMicroSecondsPositive div ?MICROSECONDS_IN_SECOND,
+    Seconds = TotalSeconds rem ?SECONDS_IN_HOUR,
+    TotalHours = TotalSeconds div ?SECONDS_IN_HOUR,
+    Hours = TotalHours rem ?HOURS_IN_DAY,
+    TotalDays = TotalHours div ?HOURS_IN_DAY,
+    if
+        TotalDays > 0 ->
+            lists:flatten([Sign,
+                           time_value_to_list(TotalDays, "day"), $ ,
+                           time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(MicroSeconds, "microsecond")]);
+        Hours > 0 ->
+            lists:flatten([Sign,
+                           time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(MicroSeconds, "microsecond")]);
+        Seconds > 0 ->
+            lists:flatten([Sign,
+                           time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(MicroSeconds, "microsecond")]);
+        true ->
+            lists:flatten([Sign,
+                           time_value_to_list(MicroSeconds, "microsecond")])
+    end;
+microseconds_to_string(_, _) ->
+    erlang:exit(badarg).
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Nanoseconds since the UNIX epoch.===
