@@ -11,7 +11,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2018 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2015-2019 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -31,13 +31,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2018 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2015-2019 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_http_rest).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -279,7 +279,7 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
         end,
         API = #api{method = HandlerMethod,
                    path = Path,
-                   parameters = lists:member($*, Prefix ++ Path),
+                   parameters = cloudi_service_name:pattern(Prefix ++ Path),
                    handler_f = Handler1,
                    arity = Arity},
         subscribe_paths(Method, Path, FormatsN, API, Lookup0, Dispatcher)
@@ -307,7 +307,7 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
             {stop, Reason, State#state{api_state = StateAPI}}
     end.
 
-cloudi_service_handle_request(_Type, Name, Pattern, RequestInfo, Request,
+cloudi_service_handle_request(_RequestType, Name, Pattern, RequestInfo, Request,
                               Timeout, Priority, TransId, _Pid,
                               #state{prefix = Prefix,
                                      lookup = Lookup,
@@ -333,11 +333,13 @@ cloudi_service_handle_request(_Type, Name, Pattern, RequestInfo, Request,
         DebugLevel =:= off ->
             ok;
         RequestInfo /= <<>> ->
-            protocol_debug_log(DebugLevel, "request ~p ~p",
-                               [Name, {RequestInfo, Request}]);
+            ?LOG(DebugLevel,
+                 "request ~p ~p",
+                 [Name, {RequestInfo, Request}]);
         true ->
-            protocol_debug_log(DebugLevel, "request ~p ~p",
-                               [Name, Request])
+            ?LOG(DebugLevel,
+                 "request ~p ~p",
+                 [Name, Request])
     end,
     true = is_list(ParametersL),
     ReturnAPI = if
@@ -350,49 +352,31 @@ cloudi_service_handle_request(_Type, Name, Pattern, RequestInfo, Request,
             ResponseInfo = response_info_headers([], Name, Format,
                                                  ContentTypes,
                                                  ContentDisposition),
-            if
-                DebugLevel =:= off ->
-                    ok;
-                true ->
-                    protocol_debug_log(DebugLevel, "response ~p ~p",
-                                       [Name, {ResponseInfo, Response}])
-            end,
+            ?LOG(DebugLevel,
+                 "response ~p ~p",
+                 [Name, {ResponseInfo, Response}]),
             {reply, ResponseInfo, Response,
              State#state{api_state = NewStateAPI}};
         {reply, ResponseInfo, Response, NewStateAPI} ->
             NewResponseInfo = response_info_headers(ResponseInfo, Name, Format,
                                                     ContentTypes,
                                                     ContentDisposition),
-            if
-                DebugLevel =:= off ->
-                    ok;
-                true ->
-                    protocol_debug_log(DebugLevel, "response ~p ~p",
-                                       [Name, {NewResponseInfo, Response}])
-            end,
+            ?LOG(DebugLevel,
+                 "response ~p ~p",
+                 [Name, {NewResponseInfo, Response}]),
             {reply, NewResponseInfo, Response,
              State#state{api_state = NewStateAPI}};
         {forward, NextName, NextRequestInfo, NextRequest, NewStateAPI} ->
-            if
-                DebugLevel =:= off ->
-                    ok;
-                true ->
-                    protocol_debug_log(DebugLevel, "forward ~p to ~p ~p",
-                                       [Name, NextName,
-                                        {NextRequestInfo, NextRequest}])
-            end,
+            ?LOG(DebugLevel,
+                 "forward ~p to ~p ~p",
+                 [Name, NextName, {NextRequestInfo, NextRequest}]),
             {forward, NextName, NextRequestInfo, NextRequest,
              State#state{api_state = NewStateAPI}};
         {forward, NextName, NextRequestInfo, NextRequest,
          NextTimeout, NextPriority, NewStateAPI} ->
-            if
-                DebugLevel =:= off ->
-                    ok;
-                true ->
-                    protocol_debug_log(DebugLevel, "forward ~p to ~p ~p",
-                                       [Name, NextName,
-                                        {NextRequestInfo, NextRequest}])
-            end,
+            ?LOG(DebugLevel,
+                 "forward ~p to ~p ~p",
+                 [Name, NextName, {NextRequestInfo, NextRequest}]),
             {forward, NextName, NextRequestInfo, NextRequest,
              NextTimeout, NextPriority,
              State#state{api_state = NewStateAPI}};
@@ -408,7 +392,7 @@ cloudi_service_handle_info(Request,
                            Dispatcher) ->
     if
         InfoF =:= undefined ->
-            ?LOG_WARN("Unknown info \"~p\"", [Request]),
+            ?LOG_WARN("Unknown info \"~w\"", [Request]),
             {noreply, State};
         true ->
             case InfoF(Request, StateAPI, Dispatcher) of
@@ -503,15 +487,3 @@ response_info_headers(ResponseInfo0, Name, Format,
 response_info_headers(ResponseInfo, _, _, _, _) ->
     ResponseInfo.
 
-protocol_debug_log(trace, Message, Args) ->
-    ?LOG_TRACE(Message, Args);
-protocol_debug_log(debug, Message, Args) ->
-    ?LOG_DEBUG(Message, Args);
-protocol_debug_log(info, Message, Args) ->
-    ?LOG_INFO(Message, Args);
-protocol_debug_log(warn, Message, Args) ->
-    ?LOG_WARN(Message, Args);
-protocol_debug_log(error, Message, Args) ->
-    ?LOG_ERROR(Message, Args);
-protocol_debug_log(fatal, Message, Args) ->
-    ?LOG_FATAL(Message, Args).
