@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2012-2020 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2012-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -31,8 +31,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2012-2020 Michael Truog
-%%% @version 2.0.1 {@date} {@time}
+%%% @copyright 2012-2021 Michael Truog
+%%% @version 2.0.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_map_reduce).
@@ -41,62 +41,119 @@
 -behaviour(cloudi_service).
 
 %% external interface
--export([resume/2,
-         resume/3,
-         suspend/2,
-         suspend/3]).
+-export([aspect_suspend/1,
+         aspect_resume/1,
+         elapsed_seconds/0]).
 
 %% cloudi_service callbacks
 -export([cloudi_service_init/4,
-         cloudi_service_handle_request/11,
          cloudi_service_handle_info/3,
          cloudi_service_terminate/3]).
 
 -include_lib("cloudi_core/include/cloudi_logger.hrl").
 -include_lib("cloudi_core/include/cloudi_service.hrl").
 
--define(DEFAULT_MAP_REDUCE_MODULE,     undefined).
--define(DEFAULT_MAP_REDUCE_ARGUMENTS,         []).
--define(DEFAULT_NAME,               "controller").
--define(DEFAULT_CONCURRENCY,                 1.0). % schedulers multiplier
--define(DEFAULT_LOG_EXECUTION_TIME,         true).
+-define(DEFAULT_MAP_REDUCE_MODULE,            undefined).
+-define(DEFAULT_MAP_REDUCE_ARGUMENTS,                []).
+-define(DEFAULT_NAME,                      "controller").
+-define(DEFAULT_CONCURRENCY,                        1.0).
+        % logical cpu count multiplier
+        % (calculated the same way as count_process and count_thread in
+        %  service configuration).
+-define(DEFAULT_LOG_EXECUTION_TIME,                true).
+-define(DEFAULT_RETRY,                                3).
+        % Max retries with a Timeout value of timeout_max
+        % before cloudi_service_map_reduce_resend is called.
+        % The Timeout value needs to be provided in the map_send_args()
+        % and is typically increasing due to service request failures.
+-define(DEFAULT_RETRY_DELAY,                          0). % milliseconds
 
 -type map_send_args() :: nonempty_list().
 -export_type([map_send_args/0]).
 
+-record(map_send,
+    {
+        send_args
+            :: map_send_args(),
+        retry_count
+            :: non_neg_integer()
+    }).
+
 -record(state,
     {
-        map_reduce_module :: module(),
-        map_reduce_state :: any(),
-        map_reduce_name :: string(),
-        map_count :: pos_integer(),
-        log_execution_time :: boolean(),
-        map_requests :: #{cloudi_service:trans_id() := map_send_args()},
-        time_running :: cloudi_timestamp:seconds_monotonic(),
-        suspended = false :: boolean(),
-        elapsed_seconds = 0 :: non_neg_integer()
+        service
+            :: pid(),
+        map_reduce_module
+            :: module(),
+        map_reduce_state
+            :: any(),
+        map_reduce_name
+            :: string(),
+        map_count
+            :: pos_integer(),
+        log_execution_time
+            :: boolean(),
+        retry
+            :: non_neg_integer(),
+        retry_delay
+            :: non_neg_integer(),
+        timeout_max
+            :: cloudi_service:timeout_value_milliseconds(),
+        map_requests
+            :: #{cloudi_service:trans_id() := #map_send{}},
+        time_running
+            :: cloudi_timestamp:seconds_monotonic(),
+        suspended = false
+            :: boolean(),
+        elapsed_seconds = 0
+            :: non_neg_integer()
+    }).
+
+-record(init_state,
+    {
+        service
+            :: pid(),
+        info_queued = []
+            :: list()
     }).
 
 -record(init_begin,
     {
-        service :: pid(),
-        prefix :: string(),
-        timeout :: cloudi_service_api:timeout_initialize_value_milliseconds(),
-        map_reduce_module :: module(),
-        map_reduce_args :: list(),
-        map_reduce_name :: string(),
-        concurrency :: number(),
-        log_execution_time :: boolean(),
-        time_start :: cloudi_timestamp:seconds_monotonic()
+        service
+            :: pid(),
+        prefix
+            :: string(),
+        timeout
+            :: cloudi_service_api:timeout_initialize_value_milliseconds(),
+        map_reduce_module
+            :: module(),
+        map_reduce_args
+            :: list(),
+        map_reduce_name
+            :: string(),
+        concurrency
+            :: number(),
+        log_execution_time
+            :: boolean(),
+        retry
+            :: non_neg_integer(),
+        retry_delay
+            :: non_neg_integer(),
+        timeout_max
+            :: cloudi_service:timeout_value_milliseconds(),
+        time_start
+            :: cloudi_timestamp:seconds_monotonic()
     }).
 
 -record(init_end,
     {
-        state :: undefined | #state{},
-        error = undefined :: any()
+        state
+            :: undefined | #state{},
+        error = undefined
+            :: any()
     }).
 
--define(INFO_RETRY_INTERVAL, 500). % milliseconds
+-define(ELAPSED_SECONDS_PDICT_KEY, cloudi_service_map_reduce_elapsed_seconds).
 
 %%%------------------------------------------------------------------------
 %%% Callback functions from behavior
@@ -145,42 +202,62 @@
 %%% External interface functions
 %%%------------------------------------------------------------------------
 
--type agent() :: cloudi:agent().
--type service_name() :: cloudi:service_name().
--type timeout_milliseconds() :: cloudi:timeout_milliseconds().
--type module_response(Result) ::
-    {{ok, Result}, AgentNew :: agent()} |
-    {{error, cloudi:error_reason()}, AgentNew :: agent()}.
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Function for aspects_suspend service configuration option.===
+%% Add as {cloudi_service_map_reduce, aspect_suspend}.
+%% @end
+%%-------------------------------------------------------------------------
 
--spec resume(Agent :: agent(),
-             Prefix :: service_name()) ->
-    module_response(ok | {error, any()}).
+-spec aspect_suspend(State :: #state{}) ->
+    {ok, #state{}}.
 
-resume(Agent, Prefix) ->
-    cloudi:send_sync(Agent, Prefix, resume).
+aspect_suspend(#state{map_reduce_module = MapReduceModule,
+                      map_reduce_name = MapReduceName,
+                      time_running = TimeRunningStart,
+                      suspended = false,
+                      elapsed_seconds = ElapsedSeconds} = State) ->
+    TimeRunningEnd = cloudi_timestamp:seconds_monotonic(),
+    ElapsedSecondsNew = ElapsedSeconds + (TimeRunningEnd - TimeRunningStart),
+    ?LOG_INFO("~s ~ts suspended",
+              [MapReduceModule, cloudi_service_name:utf8(MapReduceName)]),
+    {ok,
+     State#state{suspended = true,
+                 elapsed_seconds = ElapsedSecondsNew}}.
 
--spec resume(Agent :: agent(),
-             Prefix :: service_name(),
-             Timeout :: timeout_milliseconds()) ->
-    module_response(ok | {error, any()}).
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Function for aspects_resume service configuration option.===
+%% Add as {cloudi_service_map_reduce, aspect_resume}.
+%% @end
+%%-------------------------------------------------------------------------
 
-resume(Agent, Prefix, Timeout) ->
-    cloudi:send_sync(Agent, Prefix, resume, Timeout).
+-spec aspect_resume(State :: #state{}) ->
+    {ok, #state{}}.
 
--spec suspend(Agent :: agent(),
-              Prefix :: service_name()) ->
-    module_response(ok | {error, any()}).
+aspect_resume(#state{map_reduce_module = MapReduceModule,
+                     map_reduce_name = MapReduceName,
+                     suspended = true} = State) ->
+    TimeRunningStart = cloudi_timestamp:seconds_monotonic(),
+    ?LOG_INFO("~s ~ts resumed",
+              [MapReduceModule, cloudi_service_name:utf8(MapReduceName)]),
+    {ok,
+     State#state{time_running = TimeRunningStart,
+                 suspended = false}}.
 
-suspend(Agent, Prefix) ->
-    cloudi:send_sync(Agent, Prefix, suspend).
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Get the current elapsed seconds in a Map-Reduce callback function.===
+%% @end
+%%-------------------------------------------------------------------------
 
--spec suspend(Agent :: agent(),
-              Prefix :: service_name(),
-              Timeout :: timeout_milliseconds()) ->
-    module_response(ok | {error, any()}).
+-spec elapsed_seconds() ->
+    cloudi_service_api:seconds().
 
-suspend(Agent, Prefix, Timeout) ->
-    cloudi:send_sync(Agent, Prefix, suspend, Timeout).
+elapsed_seconds() ->
+    {TimeRunningStart,
+     ElapsedSeconds} = erlang:get(?ELAPSED_SECONDS_PDICT_KEY),
+    ElapsedSeconds + (cloudi_timestamp:seconds_monotonic() - TimeRunningStart).
 
 %%%------------------------------------------------------------------------
 %%% Callback functions from cloudi_service
@@ -192,14 +269,20 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
         {map_reduce_args,        ?DEFAULT_MAP_REDUCE_ARGUMENTS},
         {name,                   ?DEFAULT_NAME},
         {concurrency,            ?DEFAULT_CONCURRENCY},
-        {log_execution_time,     ?DEFAULT_LOG_EXECUTION_TIME}],
-    [MapReduceModule, MapReduceArgs, Name, Concurrency, LogExecutionTime] =
+        {log_execution_time,     ?DEFAULT_LOG_EXECUTION_TIME},
+        {retry,                  ?DEFAULT_RETRY},
+        {retry_delay,            ?DEFAULT_RETRY_DELAY}],
+    [MapReduceModule, MapReduceArgs, Name, Concurrency, LogExecutionTime,
+     Retry, RetryDelay] =
         cloudi_proplists:take_values(Defaults, Args),
     TimeStart = cloudi_timestamp:seconds_monotonic(),
     true = is_atom(MapReduceModule) andalso (MapReduceModule /= undefined),
     true = is_list(MapReduceArgs),
     true = is_number(Concurrency) andalso (Concurrency > 0),
     true = is_boolean(LogExecutionTime),
+    true = is_integer(Retry) andalso (Retry >= 0),
+    true = is_integer(RetryDelay) andalso
+           (RetryDelay >= 0) andalso (RetryDelay =< 4294967295),
     case application:load(MapReduceModule) of
         ok ->
             ok = cloudi_x_reltool_util:application_start(MapReduceModule,
@@ -210,110 +293,95 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
         {error, _} ->
             ok = cloudi_x_reltool_util:module_loaded(MapReduceModule)
     end,
+    TimeoutMax = cloudi_service:timeout_max(Dispatcher),
     % cloudi_service_init/4 is always executed by the service process
     Service = self(),
-    Service !  #init_begin{service = Service,
-                           prefix = Prefix,
-                           timeout = Timeout,
-                           map_reduce_module = MapReduceModule,
-                           map_reduce_args = MapReduceArgs,
-                           map_reduce_name = Name,
-                           concurrency = Concurrency,
-                           log_execution_time = LogExecutionTime,
-                           time_start = TimeStart},
-    cloudi_service:subscribe(Dispatcher, Name),
-    {ok, undefined}.
-
-cloudi_service_handle_request(_RequestType, _Name, _Pattern,
-                              _RequestInfo, Request,
-                              _Timeout, _Priority, _TransId, _Pid,
-                              State, Dispatcher) ->
-    request(Request, State, Dispatcher).
+    Service ! #init_begin{service = Service,
+                          prefix = Prefix,
+                          timeout = Timeout,
+                          map_reduce_module = MapReduceModule,
+                          map_reduce_args = MapReduceArgs,
+                          map_reduce_name = Name,
+                          concurrency = Concurrency,
+                          log_execution_time = LogExecutionTime,
+                          retry = Retry,
+                          retry_delay = RetryDelay,
+                          timeout_max = TimeoutMax,
+                          time_start = TimeStart},
+    ok = cloudi_service:subscribe(Dispatcher, Name),
+    {ok, #init_state{service = Service}}.
 
 cloudi_service_handle_info(#init_begin{service = Service} = InitBegin,
-                           undefined, Dispatcher) ->
+                           #init_state{} = InitState,
+                           Dispatcher) ->
     % cloudi_service_map_reduce_new/5 execution occurs outside of
     % cloudi_service_init/4 to allow send_sync and recv_async function calls
-    % because no Erlang process linking/spawning/etc. should be occurring,
-    % only algorithmic initialization.  Initialization is done in a temporary
-    % process so there is no blocking problem (with send_sync or recv_async)
+    % (only algorithmic initialization should be occurring).
+    % Initialization is done in a temporary process so there is
+    % no blocking problem (with send_sync or recv_async)
     % if duo_mode == true.  No timeout is enforced on
     % cloudi_service_map_reduce_new/5 execution.
-    _ = erlang:spawn_link(fun() ->
+    _ = erlang:spawn(fun() ->
+        true = erlang:link(Service),
         case init(InitBegin, Dispatcher) of
             {noreply, State} ->
                 Service ! #init_end{state = State},
                 ok;
-            {stop, Reason, State} when Reason /= undefined ->
+            {stop, Reason, State} ->
+                true = Reason /= undefined,
                 Service ! #init_end{state = State,
                                     error = Reason},
                 ok
         end,
         true = erlang:unlink(Service)
     end),
-    {noreply, undefined};
-cloudi_service_handle_info(#init_end{state = State,
-                                     error = Error},
-                           undefined, _Dispatcher) ->
-    if
-        Error =:= undefined ->
-            {noreply, State};
-        true ->
-            {stop, Error, State}
-    end;
-cloudi_service_handle_info(Request, undefined, Dispatcher) ->
-    % waiting for #init_end{} still
-    erlang:send_after(?INFO_RETRY_INTERVAL,
-                      cloudi_service:self(Dispatcher), Request),
-    {noreply, undefined};
+    {noreply, InitState};
+cloudi_service_handle_info(#init_end{} = InitEnd,
+                           #init_state{} = InitState,
+                           _Dispatcher) ->
+    init_end(InitEnd, InitState);
+cloudi_service_handle_info(Request,
+                           #init_state{info_queued = InfoQueued} = InitState,
+                           _Dispatcher) ->
+    {noreply, InitState#init_state{info_queued = [Request | InfoQueued]}};
 cloudi_service_handle_info(#timeout_async_active{trans_id = TransId} = Request,
-                           #state{map_reduce_module = MapReduceModule,
-                                  map_reduce_state = MapReduceState,
-                                  map_requests = MapRequests} = State,
+                           #state{map_requests = MapRequests} = State,
                            Dispatcher) ->
-    case maps:find(TransId, MapRequests) of
-        {ok, [_ | SendArgs]} ->
-            case MapReduceModule:
-                 cloudi_service_map_reduce_resend([Dispatcher | SendArgs],
-                                                  MapReduceState) of
-                {ok, SendArgsNew, MapReduceStateNew} ->
-                    case map_send_request(SendArgsNew,
-                                          maps:remove(TransId, MapRequests)) of
-                        {ok, MapRequestsNew} ->
-                            {noreply,
-                             State#state{map_reduce_state = MapReduceStateNew,
-                                         map_requests = MapRequestsNew}};
-                        {error, _} = Error ->
-                            {stop, Error, State}
-                    end;
-                {error, _} = Error ->
-                    {stop, Error, State}
-            end;
+    case maps:take(TransId, MapRequests) of
+        {MapRequest, MapRequestsNew} ->
+            map_retry(MapRequest,
+                      State#state{map_requests = MapRequestsNew},
+                      Dispatcher);
         error ->
-            cloudi_service_map_reduce_info(Request, State, Dispatcher)
+            map_info(Request, State, Dispatcher)
     end;
+cloudi_service_handle_info({cloudi_service_map_reduce_retry, MapRequest},
+                           State, Dispatcher) ->
+    map_resend(MapRequest, State, Dispatcher);
 cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                                                 response = Response,
                                                 timeout = Timeout,
                                                 trans_id = TransId} = Request,
                            #state{map_reduce_module = MapReduceModule,
                                   map_reduce_state = MapReduceState,
-                                  map_requests = MapRequests} = State,
+                                  map_requests = MapRequests,
+                                  time_running = TimeRunningStart,
+                                  elapsed_seconds = ElapsedSeconds} = State,
                            Dispatcher) ->
-    case maps:find(TransId, MapRequests) of
-        {ok, [_ | SendArgs]} ->
+    case maps:take(TransId, MapRequests) of
+        {#map_send{send_args = [_ | SendArgs]},
+         MapRequestsNew} ->
+            ok = elapsed_seconds_set(TimeRunningStart, ElapsedSeconds),
             case MapReduceModule:
                  cloudi_service_map_reduce_recv([Dispatcher | SendArgs],
                                                 ResponseInfo, Response,
                                                 Timeout, TransId,
                                                 MapReduceState, Dispatcher) of
                 {ok, MapReduceStateNew} ->
-                    MapRequestsNew = maps:remove(TransId, MapRequests),
                     StateNew = State#state{map_reduce_state = MapReduceStateNew,
                                            map_requests = MapRequestsNew},
                     map_check_continue(StateNew, Dispatcher);
                 {done, MapReduceStateNew} ->
-                    MapRequestsNew = maps:remove(TransId, MapRequests),
                     StateNew = State#state{map_reduce_state = MapReduceStateNew,
                                            map_requests = MapRequestsNew},
                     map_check_done(StateNew);
@@ -321,40 +389,49 @@ cloudi_service_handle_info(#return_async_active{response_info = ResponseInfo,
                     {stop, Error, State}
             end;
         error ->
-            cloudi_service_map_reduce_info(Request, State, Dispatcher)
+            map_info(Request, State, Dispatcher)
     end;
 cloudi_service_handle_info(Request, State, Dispatcher) ->
-    cloudi_service_map_reduce_info(Request, State, Dispatcher).
+    map_info(Request, State, Dispatcher).
 
-cloudi_service_terminate(_Reason, _Timeout, undefined) ->
-    ok;
 cloudi_service_terminate(shutdown, _Timeout,
                          #state{log_execution_time = true,
                                 map_requests = #{},
                                 time_running = TimeRunningStart,
+                                suspended = Suspended,
                                 elapsed_seconds = ElapsedSeconds}) ->
-    TimeRunningEnd = cloudi_timestamp:seconds_monotonic(),
-    ElapsedSecondsNew = ElapsedSeconds + (TimeRunningEnd - TimeRunningStart),
+    ElapsedSecondsNew = if
+        Suspended =:= true ->
+            ElapsedSeconds;
+        Suspended =:= false ->
+            TimeRunningEnd = cloudi_timestamp:seconds_monotonic(),
+            ElapsedSeconds + (TimeRunningEnd - TimeRunningStart)
+    end,
     ?LOG_INFO("total time taken was ~p hours",
               [hours_elapsed(ElapsedSecondsNew)]),
     ok;
-cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
+cloudi_service_terminate(_Reason, _Timeout, _State) ->
     ok.
 
 %%%------------------------------------------------------------------------
 %%% Private functions
 %%%------------------------------------------------------------------------
 
-init(#init_begin{prefix = Prefix,
+init(#init_begin{service = Service,
+                 prefix = Prefix,
                  timeout = Timeout,
                  map_reduce_module = MapReduceModule,
                  map_reduce_args = MapReduceArgs,
                  map_reduce_name = MapReduceName,
                  concurrency = Concurrency,
                  log_execution_time = LogExecutionTime,
+                 retry = Retry,
+                 retry_delay = RetryDelay,
+                 timeout_max = TimeoutMax,
                  time_start = TimeStart},
      Dispatcher) ->
     MapCount = cloudi_concurrency:count(Concurrency),
+    ok = elapsed_seconds_set(TimeStart),
     case MapReduceModule:
          cloudi_service_map_reduce_new(MapReduceArgs, MapCount,
                                        Prefix, Timeout, Dispatcher) of
@@ -363,11 +440,15 @@ init(#init_begin{prefix = Prefix,
                           MapReduceModule, MapReduceState, Dispatcher) of
                 {ok, MapRequests, MapReduceStateNew} ->
                     {noreply,
-                     #state{map_reduce_module = MapReduceModule,
+                     #state{service = Service,
+                            map_reduce_module = MapReduceModule,
                             map_reduce_state = MapReduceStateNew,
                             map_reduce_name = MapReduceName,
                             map_count = MapCount,
                             log_execution_time = LogExecutionTime,
+                            retry = Retry,
+                            retry_delay = RetryDelay,
+                            timeout_max = TimeoutMax,
                             map_requests = MapRequests,
                             time_running = TimeStart}};
                 {error, _} = Error ->
@@ -377,50 +458,61 @@ init(#init_begin{prefix = Prefix,
             {stop, Error, undefined}
     end.
 
-request(_, undefined, _Dispatcher) ->
-    {reply, {error, init_pending}, undefined};
-request(suspend,
-        #state{map_reduce_module = MapReduceModule,
-               map_reduce_name = MapReduceName,
-               time_running = TimeRunningStart,
-               suspended = false,
-               elapsed_seconds = ElapsedSeconds} = State, _Dispatcher) ->
-    TimeRunningEnd = cloudi_timestamp:seconds_monotonic(),
-    ElapsedSecondsNew = ElapsedSeconds + (TimeRunningEnd - TimeRunningStart),
-    ?LOG_INFO("~s ~ts suspended", [MapReduceModule, MapReduceName]),
-    {reply, ok,
-     State#state{suspended = true,
-                 elapsed_seconds = ElapsedSecondsNew}};
-request(suspend,
-        #state{suspended = true} = State, _Dispatcher) ->
-    {reply, {error, already_suspended}, State};
-request(resume,
-        #state{map_reduce_module = MapReduceModule,
-               map_reduce_state = MapReduceState,
-               map_reduce_name = MapReduceName,
-               map_count = MapCount,
-               map_requests = MapRequests,
-               suspended = true} = State, Dispatcher) ->
-    TimeRunningStart = cloudi_timestamp:seconds_monotonic(),
-    case map_send(MapCount - maps:size(MapRequests), MapRequests,
-                  MapReduceModule, MapReduceState, Dispatcher) of
-        {ok, MapRequestsNew, MapReduceStateNew} ->
-            ?LOG_INFO("~s ~ts resumed", [MapReduceModule, MapReduceName]),
-            {reply, ok,
-             State#state{map_reduce_state = MapReduceStateNew,
-                         map_requests = MapRequestsNew,
-                         time_running = TimeRunningStart,
-                         suspended = false}};
+init_end(#init_end{state = State,
+                   error = undefined},
+         #init_state{service = Service,
+                     info_queued = InfoQueued}) ->
+    ok = init_end_send(lists:reverse(InfoQueued), Service),
+    {noreply, State};
+init_end(#init_end{state = State,
+                   error = Error},
+         #init_state{}) ->
+    {stop, Error, State}.
+
+init_end_send([], _) ->
+    ok;
+init_end_send([Request | InfoQueued], Service) ->
+    Service ! Request,
+    init_end_send(InfoQueued, Service).
+
+map_retry(MapRequest, State, Dispatcher) ->
+    case retry(MapRequest, State) of
+        true ->
+            {noreply, State};
+        false ->
+            map_resend(MapRequest, State, Dispatcher)
+    end.
+
+map_resend(#map_send{send_args = [_ | SendArgsTail],
+                     retry_count = RetryCount},
+           #state{map_reduce_module = MapReduceModule,
+                  map_reduce_state = MapReduceState,
+                  map_requests = MapRequests,
+                  time_running = TimeRunningStart,
+                  elapsed_seconds = ElapsedSeconds} = State, Dispatcher) ->
+    SendArgs = [Dispatcher | SendArgsTail],
+    ok = elapsed_seconds_set(TimeRunningStart, ElapsedSeconds),
+    case MapReduceModule:
+         cloudi_service_map_reduce_resend(SendArgs, MapReduceState) of
+        {ok, SendArgsNew, MapReduceStateNew} ->
+            case map_send_request(SendArgsNew, RetryCount, MapRequests) of
+                {ok, MapRequestsNew} ->
+                    {noreply,
+                     State#state{map_reduce_state = MapReduceStateNew,
+                                 map_requests = MapRequestsNew}};
+                {error, _} = Error ->
+                    {stop, Error, State}
+            end;
         {error, _} = Error ->
             {stop, Error, State}
-    end;
-request(resume,
-        #state{suspended = false} = State, _Dispatcher) ->
-    {reply, {error, already_resumed}, State}.
+    end.
 
-cloudi_service_map_reduce_info(Request, State, Dispatcher) ->
+map_info(Request, State, Dispatcher) ->
     #state{map_reduce_module = MapReduceModule,
-           map_reduce_state = MapReduceState} = State,
+           map_reduce_state = MapReduceState,
+           time_running = TimeRunningStart,
+           elapsed_seconds = ElapsedSeconds} = State,
+    ok = elapsed_seconds_set(TimeRunningStart, ElapsedSeconds),
     case MapReduceModule:
          cloudi_service_map_reduce_info(Request, MapReduceState, Dispatcher) of
         {ok, MapReduceStateNew} ->
@@ -454,15 +546,20 @@ map_send(Count, MapRequests, MapReduceModule, MapReduceState, Dispatcher) ->
     end.
 
 map_send_request(SendArgs, MapRequests) ->
+    map_send_request(SendArgs, 0, MapRequests).
+
+map_send_request(SendArgs, RetryCount, MapRequests) ->
     case erlang:apply(cloudi_service, send_async_active, SendArgs) of
         {ok, TransId} ->
-            {ok, maps:put(TransId, SendArgs, MapRequests)};
+            MapRequestsNew = maps:put(TransId,
+                                      #map_send{send_args = SendArgs,
+                                                retry_count = RetryCount},
+                                      MapRequests),
+            {ok, MapRequestsNew};
         {error, _} = Error ->
             Error
     end.
 
-map_check_continue(#state{suspended = true} = State, _Dispatcher) ->
-    {noreply, State};
 map_check_continue(#state{map_reduce_module = MapReduceModule,
                           map_reduce_state = MapReduceState,
                           map_requests = MapRequests} = State, Dispatcher) ->
@@ -483,6 +580,76 @@ map_check_done(#state{map_requests = MapRequests} = State) ->
             {noreply, State}
     end.
 
+retry(#map_send{send_args = [_Dispatcher, _Name, _Request,
+                             TimeoutMax],
+                retry_count = RetryCount} = MapRequest,
+      #state{service = Service,
+             retry = Retry,
+             retry_delay = RetryDelay,
+             timeout_max = TimeoutMax}) ->
+    if
+        RetryCount < Retry ->
+            retry_delay(RetryDelay, Service, MapRequest);
+        true ->
+            erlang:exit(retry_max)
+    end;
+retry(#map_send{send_args = [_Dispatcher, _Name, _Request,
+                             TimeoutMax, _PatternPid],
+                retry_count = RetryCount} = MapRequest,
+      #state{service = Service,
+             retry = Retry,
+             retry_delay = RetryDelay,
+             timeout_max = TimeoutMax}) ->
+    if
+        RetryCount < Retry ->
+            retry_delay(RetryDelay, Service, MapRequest);
+        true ->
+            erlang:exit(retry_max)
+    end;
+retry(#map_send{send_args = [_Dispatcher, _Name, _RequestInfo, _Request,
+                             TimeoutMax, _Priority],
+                retry_count = RetryCount} = MapRequest,
+      #state{service = Service,
+             retry = Retry,
+             retry_delay = RetryDelay,
+             timeout_max = TimeoutMax}) ->
+    if
+        RetryCount < Retry ->
+            retry_delay(RetryDelay, Service, MapRequest);
+        true ->
+            erlang:exit(retry_max)
+    end;
+retry(#map_send{send_args = [_Dispatcher, _Name, _RequestInfo, _Request,
+                             TimeoutMax, _Priority, _PatternPid],
+                retry_count = RetryCount} = MapRequest,
+      #state{service = Service,
+             retry = Retry,
+             retry_delay = RetryDelay,
+             timeout_max = TimeoutMax}) ->
+    if
+        RetryCount < Retry ->
+            retry_delay(RetryDelay, Service, MapRequest);
+        true ->
+            erlang:exit(retry_max)
+    end;
+retry(_, _) ->
+    false.
+
+retry_delay(RetryDelay, Service,
+            #map_send{retry_count = RetryCount} = MapRequest) ->
+    MapRequestNew = MapRequest#map_send{retry_count = RetryCount + 1},
+    _ = erlang:send_after(RetryDelay, Service,
+                          {cloudi_service_map_reduce_retry, MapRequestNew}),
+    true.
+
+elapsed_seconds_set(TimeRunningStart) ->
+    elapsed_seconds_set(TimeRunningStart, 0).
+
+elapsed_seconds_set(TimeRunningStart, ElapsedSeconds) ->
+    _ = erlang:put(?ELAPSED_SECONDS_PDICT_KEY,
+                   {TimeRunningStart, ElapsedSeconds}),
+    ok.
+
 hours_elapsed(ElapsedSeconds) ->
     erlang:round((ElapsedSeconds / (60 * 60)) * 10) / 10.
 
