@@ -47,7 +47,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2015-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -68,8 +68,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2015-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2015-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_monitoring).
@@ -99,7 +99,6 @@
 
 %% cloudi_service callbacks
 -export([cloudi_service_init/4,
-         cloudi_service_handle_request/11,
          cloudi_service_handle_info/3,
          cloudi_service_terminate/3]).
 
@@ -271,9 +270,9 @@ add([_ | _] = ServiceConfig) ->
     end,
     case lists:keytake(options, 1, ServiceConfig) of
         {value, {_, Options}, NextServiceConfig} ->
-            NextServiceConfig ++ [{options, add_options(Type, Options)}];
+            [{options, add_options(Type, Options)} | NextServiceConfig];
         false ->
-            ServiceConfig ++ [{options, add_options(Type, [])}]
+            [{options, add_options(Type, [])} | ServiceConfig]
     end.
 
 %%-------------------------------------------------------------------------
@@ -620,13 +619,17 @@ cloudi_service_init(Args, _Prefix, _Timeout, Dispatcher) ->
         ErlangInterval =:= undefined ->
             ok;
         is_integer(ErlangInterval) ->
-            erlang:send_after(ErlangInterval * 1000, Service, erlang_update)
+            _ = erlang:send_after(ErlangInterval * 1000, Service,
+                                  erlang_update),
+            ok
     end,
     if
         Interval =:= undefined ->
             ok;
         is_integer(Interval) ->
-            erlang:send_after(Interval * 1000, Service, cloudi_update)
+            _ = erlang:send_after(Interval * 1000, Service,
+                                  cloudi_update),
+            ok
     end,
     ok = monitor_nodes(true, cloudi_x_cpg_app:listen_type()),
     {ok, #state{service = Service,
@@ -644,12 +647,6 @@ cloudi_service_init(Args, _Prefix, _Timeout, Dispatcher) ->
                 nodes_hidden = erlang:length(erlang:nodes(hidden)),
                 nodes_all = erlang:length(erlang:nodes(connected))}}.
 
-cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, _Request,
-                              _Timeout, _Priority, _TransId, _Pid,
-                              #state{} = State,
-                              _Dispatcher) ->
-    {reply, <<>>, State#state{}}.
-
 cloudi_service_handle_info(cloudi_update,
                            #state{service = Service,
                                   environment = EnvironmentLookup,
