@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2017-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2017-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2017-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2017-2021 Michael Truog
+%%% @version 2.0.3 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_null).
@@ -48,10 +48,12 @@
 
 -define(DEFAULT_DEBUG,                      false). % log output for debugging
 -define(DEFAULT_DEBUG_LEVEL,                trace).
+-define(DEFAULT_DEBUG_CONTENTS,             false).
 
 -record(state,
     {
-        debug_level :: off | trace | debug | info | warn | error | fatal
+        debug_level :: off | trace | debug | info | warn | error | fatal,
+        debug_contents :: boolean()
     }).
 
 %%%------------------------------------------------------------------------
@@ -65,16 +67,18 @@
 cloudi_service_init(Args, _Prefix, _Timeout, Dispatcher) ->
     Defaults = [
         {debug,                    ?DEFAULT_DEBUG},
-        {debug_level,              ?DEFAULT_DEBUG_LEVEL}],
-    [Debug, DebugLevel] = cloudi_proplists:take_values(Defaults, Args),
-    true = ((Debug =:= true) orelse
-            (Debug =:= false)),
+        {debug_level,              ?DEFAULT_DEBUG_LEVEL},
+        {debug_contents,           ?DEFAULT_DEBUG_CONTENTS}],
+    [Debug, DebugLevel,
+     DebugContents] = cloudi_proplists:take_values(Defaults, Args),
+    true = is_boolean(Debug),
     true = ((DebugLevel =:= trace) orelse
             (DebugLevel =:= debug) orelse
             (DebugLevel =:= info) orelse
             (DebugLevel =:= warn) orelse
             (DebugLevel =:= error) orelse
             (DebugLevel =:= fatal)),
+    true = is_boolean(DebugContents),
     cloudi_service:subscribe(Dispatcher, ""),
     DebugLogLevel = if
         Debug =:= false ->
@@ -82,34 +86,29 @@ cloudi_service_init(Args, _Prefix, _Timeout, Dispatcher) ->
         Debug =:= true ->
             DebugLevel
     end,
-    {ok, #state{debug_level = DebugLogLevel}}.
+    {ok, #state{debug_level = DebugLogLevel,
+                debug_contents = DebugContents}}.
 
-cloudi_service_handle_request(_Type, Name, _Pattern, _RequestInfo, _Request,
+cloudi_service_handle_request(_RequestType, Name, _Pattern,
+                              RequestInfo, Request,
                               _Timeout, _Priority, _TransId, _Pid,
-                              #state{debug_level = DebugLogLevel} = State,
+                              #state{debug_level = DebugLogLevel,
+                                     debug_contents = DebugContents} = State,
                               _Dispatcher) ->
-    debug_log(DebugLogLevel, "\"~ts\" consumed", [Name]),
+    if
+        DebugContents =:= true ->
+            ?LOG(DebugLogLevel,
+                 "\"~ts\" consumed~n"
+                 "(~tp,~n"
+                 " ~tp)",
+                 [cloudi_service_name:utf8(Name), RequestInfo, Request]);
+        DebugContents =:= false ->
+            ?LOG(DebugLogLevel,
+                 "\"~ts\" consumed",
+                 [cloudi_service_name:utf8(Name)])
+    end,
     {reply, <<>>, State}.
 
 cloudi_service_terminate(_Reason, _Timeout, _State) ->
     ok.
 
-%%%------------------------------------------------------------------------
-%%% Private functions
-%%%------------------------------------------------------------------------
-
-debug_log(off, _, _) ->
-    ok;
-debug_log(trace, Message, Args) ->
-    ?LOG_TRACE(Message, Args);
-debug_log(debug, Message, Args) ->
-    ?LOG_DEBUG(Message, Args);
-debug_log(info, Message, Args) ->
-    ?LOG_INFO(Message, Args);
-debug_log(warn, Message, Args) ->
-    ?LOG_WARN(Message, Args);
-debug_log(error, Message, Args) ->
-    ?LOG_ERROR(Message, Args);
-debug_log(fatal, Message, Args) ->
-    ?LOG_FATAL(Message, Args).
-
