@@ -11,7 +11,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -32,8 +32,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @copyright 2011-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_pqueue2).
@@ -467,15 +467,17 @@ merge({P, HL1, HR1, element, Value}, {P, HL2, HR2, queue, Queue}) ->
 -ifdef(TEST).
 -include_lib("eunit/include/eunit.hrl").
 
-internal_test_() ->
-    [
-        {"internal tests", ?_assertEqual(ok, test())}
-    ].
+-include("cloudi_x_pqueue_test.hrl").
 
-proper_test_() ->
-    {timeout, 600, [
-        {"proper tests", ?_assert(cloudi_x_pqueue_proper:qc_pq2())}
+module_test_() ->
+    {timeout, ?TEST_TIMEOUT, [
+        {"internal tests", ?_assertOk(test())}
     ]}.
 
+long_test_() ->
+    test_condition([
+        {"proper tests", ?_assert(cloudi_x_pqueue_proper:qc_pq2())}
+    ], ?CLOUDI_LONG_TEST_TIMEOUT).
+
 -endif.
 
