@@ -26,7 +26,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2017-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -47,8 +47,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2017-2020 Michael Truog
-%%% @version 2.0.1 {@date} {@time}
+%%% @copyright 2017-2021 Michael Truog
+%%% @version 2.0.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(quickrand_hash).
@@ -66,6 +66,30 @@
          jenkins64_32/1,
          jenkins64_32/2]).
 
+-ifdef(OTP_RELEASE).
+% able to use -if/-elif here
+-if(?OTP_RELEASE >= 24).
+-define(ERLANG_OTP_VERSION_24_FEATURES, true).
+-endif.
+-endif.
+
+-ifdef(ERLANG_OTP_VERSION_24_FEATURES).
+% iodata() recursive type is not infinitely recursive
+% like within the function iodata_to_list/1
+% (unable to only use no_underspecs on iodata_to_list/1)
+-dialyzer({no_underspecs,
+           [jenkins_32/1,
+            jenkins_32/2,
+            jenkins_64/1,
+            jenkins_64/2,
+            jenkins64_128/1,
+            jenkins64_128/2,
+            jenkins64_64/1,
+            jenkins64_64/2,
+            jenkins64_32/1,
+            jenkins64_32/2]}).
+-endif.
+
 % a constant which:
 %  * is not zero
 %  * is odd
