@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2017-2019 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2017-2021 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -26,36 +26,22 @@
 
 package org.cloudi.tests.count;
 
+import java.util.HashMap;
+import java.util.List;
 import com.ericsson.otp.erlang.OtpErlangPid;
 import org.cloudi.API;
 
 public class Task implements Runnable
 {
     private API api;
+    private final int thread_index;
     private long count;
      
     public Task(final int thread_index)
     {
-        try
-        {
-            this.api = new API(thread_index);
-            this.count = 0;
-        }
-        catch (API.InvalidInputException e)
-        {
-            e.printStackTrace(API.err);
-            System.exit(1);
-        }
-        catch (API.MessageDecodingException e)
-        {
-            e.printStackTrace(API.err);
-            System.exit(1);
-        }
-        catch (API.TerminateException e)
-        {
-            API.err.println("terminate count java (before init)");
-            System.exit(1);
-        }
+        this.api = null;
+        this.thread_index = thread_index;
+        this.count = 0;
     }
 
     public Object request(Integer request_type, String name, String pattern,
@@ -76,8 +62,10 @@ public class Task implements Runnable
         }
         API.out.println("count == " + this.count + " java");
         String response = Long.toString(this.count);
+        byte[] response_info = API.info_key_value_new(
+            new HashMap<String, List<String>>());
         this.api.return_(request_type, name, pattern,
-                         ("").getBytes(), response.getBytes(),
+                         response_info, response.getBytes(),
                          timeout, trans_id, pid);
         return null;
     }
@@ -86,6 +74,8 @@ public class Task implements Runnable
     {
         try
         {
+            this.api = new API(this.thread_index);
+
             // possible with Java >= 8
             //this.api.subscribe("java/get", this::request);
 
