@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -36,28 +36,12 @@ import org.cloudi.API;
 public class Task implements Runnable
 {
     private API api;
+    private final int thread_index;
      
     public Task(final int thread_index)
     {
-        try
-        {
-            this.api = new API(thread_index);
-        }
-        catch (API.InvalidInputException e)
-        {
-            e.printStackTrace(API.err);
-            System.exit(1);
-        }
-        catch (API.MessageDecodingException e)
-        {
-            e.printStackTrace(API.err);
-            System.exit(1);
-        }
-        catch (API.TerminateException e)
-        {
-            API.err.println("terminate http_req java (before init)");
-            System.exit(1);
-        }
+        this.api = null;
+        this.thread_index = thread_index;
     }
 
     public Object request(Integer request_type, String name, String pattern,
@@ -107,6 +91,7 @@ public class Task implements Runnable
     {
         try
         {
+            this.api = new API(this.thread_index);
             assert this.api.subscribe_count("java.xml/get") == 0;
 
             // possible with Java >= 8
