@@ -5,7 +5,7 @@
  
   MIT License
 
-  Copyright (c) 2017 Michael Truog <mjtruog at protonmail dot com>
+  Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -27,7 +27,7 @@
  
  *)
 
-let request type_ name pattern _ request timeout _ trans_id pid _ api =
+let request request_type name pattern _ request timeout _ trans_id pid _ api =
   let http_qs = Cloudi.info_key_value_parse request in
   let value = try Some (int_of_string (List.hd (Hashtbl.find http_qs "value")))
   with _ -> None in
@@ -37,9 +37,11 @@ let request type_ name pattern _ request timeout _ trans_id pid _ api =
   | Some (value) ->
     let s = string_of_int value in
     "<http_test><value>" ^ s ^ "</value></http_test>"
-  in
+  and http_response_headers = Hashtbl.create 1 in
+  Hashtbl.add http_response_headers "content-type" ["text/xml; charset=utf-8"] ;
+  let response_info = Cloudi.info_key_value_new http_response_headers in
   match Cloudi.return_ api
-    type_ name pattern "" response timeout trans_id pid with
+    request_type name pattern response_info response timeout trans_id pid with
   | Error (error) ->
     Cloudi.NullError (error)
   | Ok _ ->
