@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2012-2020 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2012-2021 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -37,29 +37,11 @@ import org.cloudi.API;
 public class Task implements Runnable
 {
     private API api;
-    private int thread_index;
+    private final int thread_index;
      
     public Task(final int thread_index)
     {
-        try
-        {
-            this.api = new API(thread_index);
-        }
-        catch (API.InvalidInputException e)
-        {
-            e.printStackTrace(API.err);
-            System.exit(1);
-        }
-        catch (API.MessageDecodingException e)
-        {
-            e.printStackTrace(API.err);
-            System.exit(1);
-        }
-        catch (API.TerminateException e)
-        {
-            API.err.println("terminate messaging java (before init)");
-            System.exit(1);
-        }
+        this.api = null;
         this.thread_index = thread_index;
     }
 
@@ -628,6 +610,7 @@ public class Task implements Runnable
     {
         try
         {
+            this.api = new API(this.thread_index);
             this.api.subscribe("a/b/c/d", this, "sequence1_abcd");
             this.api.subscribe("a/b/c/*", this, "sequence1_abc_");
             this.api.subscribe("a/b/*/d", this, "sequence1_ab_d");
